summaryrefslogtreecommitdiff
path: root/src/elements
diff options
context:
space:
mode:
authorSimon Robertshaw <simon@hardwired.org.uk>2012-04-17 21:38:27 (GMT)
committer Simon Robertshaw <simon@hardwired.org.uk>2012-04-17 21:38:27 (GMT)
commitffa5ccb6660d77662825db74a2de1b1ab45099c3 (patch)
treee197980ab13949884c8a46fc5bdd5e5ae8cf91a9 /src/elements
parenta7e1774d41f6b3f062f16cde5352b400a7585ce8 (diff)
downloadpowder-ffa5ccb6660d77662825db74a2de1b1ab45099c3.zip
powder-ffa5ccb6660d77662825db74a2de1b1ab45099c3.tar.gz
Alterations to TRON and fancy effects
Diffstat (limited to 'src/elements')
-rw-r--r--src/elements/tron.c64
1 files changed, 53 insertions, 11 deletions
diff --git a/src/elements/tron.c b/src/elements/tron.c
index da301fd..83d293a 100644
--- a/src/elements/tron.c
+++ b/src/elements/tron.c
@@ -12,14 +12,16 @@
* - +<----+---->+ - -
* - - - - H - - - - -
* Where H is the head with tail length 4, it checks the + area to see if it can hit any of the edges, then it is called safe, or picks the biggest area if none safe.
* .tmp bit values: 0 tail , 1 head , 2 no tail growth , 4-8 is direction , 16 is wait flag
* .tmp2 is tail length (gets longer every few hundred frames)
* .life is the timer that kills the end of the tail (the head uses life for how often it grows longer)
+ * .ctype Contains the colour, lost on save, regenerated using hue tmp (bits 7 - 16)
*/
#define TRON_HEAD 1
#define TRON_NOGROW 2
-#define TRON_WAIT 16 //it was just created, so WAIT a frame
-#define TRON_NODIE 32
+#define TRON_WAIT 4 //it was just created, so WAIT a frame
+#define TRON_NODIE 8
+#define TRON_DEATH 16 //Crashed, now dying
int tron_rx[4] = {-1, 0, 1, 0};
int tron_ry[4] = { 0,-1, 0, 1};
int new_tronhead(int x, int y, int i, int direction)
@@ -33,13 +35,13 @@ int new_tronhead(int x, int y, int i, int direction)
parts[i].life = 5;
}
//give new head our properties
- parts[np].tmp = 1 | direction<<2 | parts[i].tmp&(TRON_NOGROW|TRON_NODIE);
+ parts[np].tmp = 1 | direction<<5 | parts[i].tmp&(TRON_NOGROW|TRON_NODIE) | (parts[i].tmp&0xF800);
if (np > i)
parts[np].tmp |= TRON_WAIT;
+ parts[np].ctype = parts[i].ctype;
parts[np].tmp2 = parts[i].tmp2;
parts[np].life = parts[i].life + 2;
- parts[np].dcolour = parts[i].dcolour;
return 1;
}
int trymovetron(int x, int y, int dir, int i, int len)
@@ -88,15 +90,23 @@ int trymovetron(int x, int y, int dir, int i, int len)
}
int update_TRON(UPDATE_FUNC_ARGS) {
int r, rx, ry, np;
+ if(!parts[i].ctype)
+ {
+ int r, g, b;
+ int hue = (parts[i].tmp&0xF800)>>7;
+ HSV_to_RGB(hue,255,255,&r,&g,&b);
+ parts[i].ctype = r<<16 | g<<8 | b;
+ //Use photon-like wavelength?
+ }
if (parts[i].tmp&TRON_WAIT)
{
- parts[i].tmp -= TRON_WAIT;
+ parts[i].tmp &= ~TRON_WAIT;
return 0;
}
if (parts[i].tmp&TRON_HEAD)
{
int firstdircheck = 0,seconddir,seconddircheck = 0,lastdir,lastdircheck = 0;
- int direction = (parts[i].tmp>>2 & 0x3);
+ int direction = (parts[i].tmp>>5 & 0x3);
int originaldir = direction;
//random turn
@@ -133,20 +143,52 @@ int update_TRON(UPDATE_FUNC_ARGS) {
//now try making new head, even if it fails
if (new_tronhead(x + tron_rx[direction],y + tron_ry[direction],i,direction) == -1)
{
- //ohgod crash, <sparkle effect start here>
+ //ohgod crash
+ parts[i].tmp |= TRON_DEATH;
//trigger tail death for TRON_NODIE, or is that mode even needed? just set a high tail length(but it still won't start dying when it crashes)
}
//set own life and clear .tmp (it dies if it can't move anyway)
parts[i].life = parts[i].tmp2;
- parts[i].tmp = 0;
+ parts[i].tmp &= parts[i].tmp&0xF810;
}
else // fade tail deco, or prevent tail from dieing
{
if (parts[i].tmp&TRON_NODIE)
parts[i].life++;
- parts[i].dcolour = clamp_flt((float)parts[i].life/(float)parts[i].tmp2,0,1.0f) << 24 | parts[i].dcolour&0x00FFFFFF;
+ //parts[i].dcolour = clamp_flt((float)parts[i].life/(float)parts[i].tmp2,0,1.0f) << 24 | parts[i].dcolour&0x00FFFFFF;
}
-
return 0;
}
+
+int graphics_TRON(GRAPHICS_FUNC_ARGS) {
+ if(cpart->tmp & TRON_HEAD)
+ *pixel_mode |= PMODE_GLOW;
+ if(cpart->ctype)
+ {
+ *colr = (cpart->ctype & 0xFF0000)>>16;
+ *colg = (cpart->ctype & 0x00FF00)>>8;
+ *colb = (cpart->ctype & 0x0000FF);
+ }
+ else
+ {
+ *colr = 255;
+ *colg = 255;
+ *colb = 255;
+ }
+ if(cpart->tmp & TRON_DEATH)
+ {
+ *pixel_mode |= FIRE_ADD | PMODE_FLARE;
+ *firer = *colr;
+ *fireg = *colg;
+ *fireb = *colb;
+ *firea = 255;
+ }
+ if(cpart->life < cpart->tmp2)
+ {
+ *pixel_mode |= PMODE_BLEND;
+ *pixel_mode &= ~PMODE_FLAT;
+ *cola = (int)((((float)cpart->life)/((float)cpart->tmp2))*255.0f);
+ }
+ return 0;
+} \ No newline at end of file
@@ -12,14 +12,16 @@
* - - - - H - - - - -
* Where H is the head with tail length 4, it checks the + area to see if it can hit any of the edges, then it is called safe, or picks the biggest area if none safe.
- * .tmp bit values: 0 tail , 1 head , 2 no tail growth , 4-8 is direction , 16 is wait flag
+ * .tmp bit values: 1st head, 2nd no tail growth, 3rd wait flag, 4th Nodie, 5th Dying, 6th & 7th is direction, 8th - 16th hue
* .tmp2 is tail length (gets longer every few hundred frames)
* .life is the timer that kills the end of the tail (the head uses life for how often it grows longer)